/* eslint-disable no-undef */

import { Stream } from "stream";

/**
 * 
 * @returns {Promise<Stream>}
 */
async function getVcdStream() {
    const ostream = await window.getVcdStream();
    return ostream;
}

/**
 * @returns {Promise<Uint8Array>}
 */
async function readVcdFile() {
    const response = await fetch('test.vcd');
    const blob = await response.blob();
    const reader = new FileReader();
    return new Promise((resolve, reject) => {
        reader.onload = event => {
            const fileContent = event.target.result;
            const unintarray = new Uint8Array(fileContent);
            resolve(unintarray);     
        };
        reader.readAsArrayBuffer(blob);
    });
}

function debounceWrapper(fn, delay) {
    let timer
    return function () {
        if (timer) {
            clearTimeout(timer)
        }
        timer = setTimeout(() => {
            fn()
        }, delay)
    }
}


function makeIconClass(mod) {
    switch (mod.type) {
        case 'module':      return 'icon-memory-chip';
        case 'begin':       return 'icon-brackets';
        case 'fork':        return 'icon-fork';
        case 'function':    return 'icon-Function';
        case 'task':        return 'icon-task-';

        case 'event':       return 'icon-prevent';
        case 'integer':     return 'icon-integer';
        case 'parameter':   return 'icon-parameter';
        case 'real':        return 'icon-R';
        case 'realtime':    return 'icon-wave-square';
        case 'reg':         return 'icon-register';
        case 'supply0':     return 'icon-wave-square';
        case 'supply1':     return 'icon-wave-square';
        case 'time':        return 'icon-wave-square';
        case 'tri':         return 'icon-wave-square';
        case 'triand':      return 'icon-wave-square';
        case 'trior':       return 'icon-wave-square';
        case 'trireg':      return 'icon-wave-square';
        case 'tri0':        return 'icon-wave-square';
        case 'tri1':        return 'icon-wave-square';
        case 'wand':        return 'icon-wave-square';
        case 'wire':        return 'icon-wave-square';
        case 'wor':         return 'icon-wave-square';
        case 'string':      return 'icon-String';
        default:
            break;
    }
}


/**
 * @param { string } searchString
 * @param {{
 * kind: 'var' | 'scope',
 * type: 'wire' | 'reg' | 'module',
 * name: string,
 * size?: number,
 * link?: string
 * }} module 
 * @param { Set<string> } searchScope
 * @param { boolean } caseSensitivity
 * @param { boolean } displayParentOnly
 * @returns { { htmlString: string, module } | null }
*/
function makeSearchResultItem(searchString, module, searchScope, caseSensitivity, displayParentOnly) {
    if (searchScope.has(module.type)) {
        let pattern = module.name;
        if (!caseSensitivity) {
            pattern = pattern.toLowerCase();
            searchString = searchString.toLowerCase();
        }
        if (pattern.includes(searchString)) {
            let p = module;

            const deps = [];
            while (p) {
                if (p.name && p.type) {
                    deps.push(p);
                }
                p = p.parent;
                if (displayParentOnly && deps.length == 2) {
                    break;
                }
            }

            let htmlString = '';
            let depString = '';
            let currentCounts = 0;
            for (let i = deps.length - 1; i >= 0; -- i) {
                const mod = deps[i];
                // const displayName = mod.name.length > 6 ? mod.name.substring(0, 6) + '...' : mod.name;
                const iconClass = makeIconClass(mod);
                const iconText = `<span class="iconfont ${iconClass}"></span>&ensp;${mod.name}`;
                
                depString += iconText;
                htmlString += iconText;

                if (i > 0) {
                    depString += '<br>';
                    htmlString += '<div class="dep-arrow"></div>';
                }
            }

            return {
                htmlString,
                module,
                depString
            };
        } 
    }

    return null;
}

/**
 * 
 * @param {string} timescale 
 * @return {{
 *  scale: number,
 *  unit: string
 * }}
 */
function handleTimeScale(timescale) {
    const match = timescale.match(/^(\d+)(.*)$/);
    if (match.length === 3) {
        const scale = parseInt(match[1]);
        const unit = match[2];
        return { scale, unit };
    } else {
        console.log('Error: fail to parse timescale ' + timescale);
        return { scale: null, unit: null };
    }
}

const scopes = new Set(['begin', 'fork', 'function', 'module', 'task']);
const variables = new Set(['event', 'integer', 'parameter', 'real', 'realtime', 'reg', 'supply0',
                           'supply1', 'time', 'tri', 'triand', 'trior', 'trireg', 'tri0', 'tri1',
                           'wand', 'wire', 'wor', 'string']);

function isScope(wire) {
    return scopes.has(wire.type);
}

function isVariable(wire) {
    return variables.has(wire.type);
}


export {
    getVcdStream,
    readVcdFile,
    debounceWrapper,
    makeSearchResultItem,
    handleTimeScale,
    isScope,
    isVariable,
    makeIconClass,
    scopes,
    variables
};